<?php

namespace App\Http\Controllers;

use App\Http\Requests\BlfSettingsRequest;
use App\Http\Requests\UpdateIceServersRequest;
use App\Models\AppSetting;

class AppController extends Controller
{
    public function getVersion () {
        try {
            return response()->json([
                'version' => (new \App\Utils\AppStatus)->getVersion('vitxi')
            ]);
        } catch (\Exception $exception) {
            return response()->json($exception);
        }
    }

    public function getIceServers (): \Illuminate\Http\JsonResponse
    {
        try {
            return response()->json(AppSetting::getSetting('network', 'iceServers')->first());
        } catch (\Exception $exception) {
            return response()->json($exception->getMessage());
        }
    }

    public function updateIceServers (UpdateIceServersRequest $request): \Illuminate\Http\JsonResponse
    {
        $validated = $request->validated();

        try {
            $iceServers = $validated['iceServers'];

            AppSetting::where('key', 'iceServers')->delete();

            $setting = AppSetting::create([
                'type' => 'network',
                'key' => 'iceServers',
                'value' => $iceServers
            ]);

            $eventLog = new PbxApiController();
            $user = auth()->user();

            $eventLog->saveEvents(
                $user->tenant_id,
                'Ice Servers',
                'update',
                [
                    'iceServers' => $request->iceServers
                ],
                $user->user_name,
                $user->id,
                [
                ],
                $request->ip(),
                $request->userAgent()
            );

            return response()->json(
                $setting
            );
        } catch (\Exception $exception) {
            return response()->json($exception->getMessage());
        }
    }

    public function getBlfSettings (): \Illuminate\Http\JsonResponse
    {
        try {
            $settings = AppSetting::where('type', 'blf')->get();
            $keyed = $settings->mapWithKeys(function ($item, $key) {
                return [$item['key'] => $item['value']];
            });
            return response()->json($keyed);
        } catch (\Exception $e) {
            return response()->json($e->getMessage(), $e->getCode());
        }
    }

    public function updateBlfSettings (BlfSettingsRequest $request): \Illuminate\Http\JsonResponse
    {
        $validated = $request->validated();

        try {

            $setting = AppSetting::updateOrCreate(
                ['type' => 'blf', 'key' => 'limit'],
                ['value' => $validated['limit']]
            );

            $eventLog = new PbxApiController();
            $user = auth()->user();
            $eventLog->saveEvents(
                $user->tenant_id,
                'BLF Settings',
                'update',
                [
                    'limit' => $request->limit
                ],
                $user->user_name,
                $user->id,
                [
                ],
                $request->ip(),
                $request->userAgent()
            );

            return response()->json([
                'message' => __('settings_update_success'),
                'setting' => $setting
            ]);
        } catch (\Exception $e) {
            return response()->json($e->getMessage(), $e->getCode());
        }
    }
}
