<?php

namespace App\Http\Controllers;

use App\Helpers\Helper;
use App\Utils\Environment;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\URL;
use phpDocumentor\Reflection\Types\Self_;

class BrandingController extends Controller
{
    const DEFAULT_APP_NAME = 'VitXi WebRTC';

    const DEFAULT_LOGO = '/images/logo.png';

    const DEFAULT_BRAND_ICON = 'images/favicon.png';

    const CONF_FILE_PATH = '/etc/vitalpbx/addons.conf';

    public static function index () {
        // default
        $default_logo = null;
        $default_brand_icon = null;
        // $branding = false;
        // custom branding from pbx
        $app_name = null;
        $logo = null;
        $brand_icon = null;

        if (Cache::has('branding')) {
            return response()->json(Cache::get('branding'));
        }

        $currentPlan = ( new PbxApiController )->getCurrentPlan();

        // set a generic name for the application
        if (env('APP_NAME') !== 'WebRTCApp') {
            Environment::envVarUpdate('APP_NAME', 'WebRTCApp');
        }

        if (($currentPlan !== 'carrier-plus' && $currentPlan !== 'vpbx-one') || !file_exists(self::CONF_FILE_PATH)) {
            return response()->json(self::defaultBrand());
        }

        try {
            $config = parse_ini_file(self::CONF_FILE_PATH, true);
            if (array_key_exists('vitxi', $config)) {
                $payload = $config['vitxi'];

                if (trim($payload['name']) !== '') {
                    $app_name = strtolower($payload['name']) === 'vitxi' ? self::DEFAULT_APP_NAME : $payload['name'];
                    $manifest = self::getManifest();
                    if (!is_null($manifest) && isset($manifest['name']) && $app_name !== $manifest['name']) {
                        // update frontend manifest
                        self::updateManifest($app_name);
                    }
                } else {
                    $app_name = self::DEFAULT_APP_NAME;
                }

                // build the pbx url base
                $pbx_host = config('app.pbx.host') === 'localhost' ? Helper::getClientHostName() : config('app.pbx.host');
                $pbxHttpsPort = config('app.pbx.https_port');

                $protocol = 'https://';
                $pbx_base = $protocol . $pbx_host;

                //Avoid appending the port when is the default one(80 or 443)
                $pbx_base .= (
                    trim($pbxHttpsPort) !== '' && $pbxHttpsPort !== '443' ? ":$pbxHttpsPort" : ''
                );

                if (trim($payload['logo']) !== '') {
                    $logo = $pbx_base . trim($payload['logo']);
                } else {
                    $default_logo = self::DEFAULT_LOGO;
                }

                if (trim($payload['mobile_logo']) !== '') {
                    $brand_icon = $pbx_base . trim($payload['mobile_logo']);
                } else {
                    $default_brand_icon = self::DEFAULT_BRAND_ICON;
                }
                $expirationTime = 48 * 60 * 60; // 48 hours in seconds

                Cache::remember('branding', $expirationTime, static function () use ($brand_icon, $logo, $default_brand_icon, $default_logo, $app_name) {
                    return [
                        'app_name' => $app_name,
                        'default_logo' => $default_logo,
                        'default_brand_icon' => $default_brand_icon,
                        'logo' => $logo,
                        'brand_icon' => $brand_icon
                    ];
                });

                return response()->json(Cache::get('branding'));
            }

        } catch (\Exception $exception) {
            return response()->json($exception->getMessage());
        }
    }
    public static function defaultBrand () {
        return (object) [
            'app_name' => self::DEFAULT_APP_NAME,
            'default_logo' => self::DEFAULT_LOGO,
            'default_brand_icon' => self::DEFAULT_BRAND_ICON
        ];
    }

    public static function getManifest() {
        if (file_exists('/usr/share/vitxi/frontend/dist/manifest.json')) {
            $manifest = file_get_contents('/usr/share/vitxi/frontend/dist/manifest.json');
            return json_decode($manifest, true);
        }
        return null;
    }

    public static function updateManifest ($app_name = 'VitXi') {
        try {
            if (file_exists('/usr/share/vitxi/frontend/dist/manifest.json')) {
                $manifest = file_get_contents('/usr/share/vitxi/frontend/dist/manifest.json');
                $manifestDecoded = json_decode($manifest, true);
                $manifestDecoded['name'] = $app_name;
                $manifestDecoded['short_name'] = $app_name;
                $manifestDecoded['description'] = $app_name . ' is a full WebRTC client with all of the features you need.';
                $manifest = json_encode($manifestDecoded);
                file_put_contents('/usr/share/vitxi/frontend/dist/manifest.json', $manifest);
            }
        } catch (\Exception $exception) {
            // nothing to do
        }
    }
}
