<?php

namespace App\Http\Controllers;

use App\Http\Requests\SetupDatabaseRequest;
use App\Utils\Console;
use App\Utils\Environment;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Database\QueryException;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Str;
use PDO;

class DatabaseSettingsController extends Controller
{
    /**
     * This function helps to allow the frontend to verify if the application has been installed.
     * The path that calls this function is applied by a middleware that performs the verification logic.
     * In case the middleware determines that the app has not been installed, it will enter this function by sending a 200 code to the frontend.
     *
     * @return Application|ResponseFactory|Response
     */
   public function index () {
       return response(null, 200);
   }
   public function setup (SetupDatabaseRequest $request) {

       $validated = $request->validated();

       $app_password = Str::random(20);

       $configurations = (object) [
           'root_user' => $validated['username'],
           'root_password' => $validated['password'] ?: '',
           'app_user_password' => $app_password
       ];
       $configurations = json_encode($configurations);

       Console::execute('vitxi', "install {$configurations}", $result);

       $result = str_replace("\n", "", $result);

       if ($result === "1") {

           $updated = Environment::updateSettings([
               'DB_HOST' => '127.0.0.1',
               'DB_PORT' => '3306',
               'DB_DATABASE' => 'vitxi',
               'DB_USERNAME' => 'vitxi_user',
               'DB_PASSWORD' => $app_password
           ]);

           if (!$updated) {
               return response()->json(['message' => __('could_not_update')], 422);
           }

           config([
               'database.connections.mysql.host' => '127.0.0.1',
               'database.connections.mysql.port' => '3306',
               'database.connections.mysql.database' => 'vitxi',
               'database.connections.mysql.username' => 'vitxi_user',
               'database.connections.mysql.password' => $app_password
           ]);
           Artisan::call('config:clear');
           Artisan::call('config:cache');
           Artisan::call('config:cache');

           try {
               $db = new PDO('mysql:host=localhost;dbname=vitxi', 'vitxi_user', $app_password);
               Artisan::call('migrate --force');
               Artisan::call('websockets:restart');

               return response()->json([
                   'connected' => true
               ]);
           }catch(QueryException $e){
               return response()->json([
                   'message' => $e->getMessage()
               ], 422);
           }

       }

       return response()->json([
           'connected' => $result
       ], 422);
    }
}
