<?php

namespace App\Http\Controllers;
use App\Mail\SendMail;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;
use Symfony\Component\HttpFoundation\Response;


class PasswordResetRequestController extends Controller {

    public function sendPasswordResetEmail(Request $request){
        try {
            // If email does not exist
            if(!$this->validEmail($request->email)) {
                return response()->json([
                    'message' => __('email_does_not_exist')
                ], Response::HTTP_NOT_FOUND);
            } else {
                // If email exists
                $this->sendMail($request->email);
                return response()->json([
                    'message' => __('you_will_receive_a_password_recovery_link_at_your_email_address_in_a_few_minutes')
                ], Response::HTTP_OK);
            }
        } catch (\Swift_TransportException $exception) {
            return response()->json([
                'message' => __('the_link_to_reset_your_password_could_not_be_sent_please_contact_the_administrator')
            ], Response::HTTP_BAD_GATEWAY);
        }
    }


    public function sendMail($email){
        $token = $this->generateToken($email);
        Mail::to($email)->queue(new SendMail($token));
    }

    public function validEmail($email) {
        return !!User::where('email', $email)->first();
    }

    public function generateToken($email){
        $isOtherToken = DB::table('password_resets')->where('email', $email)->first();

        if($isOtherToken) {
            return $isOtherToken->token;
        }

        $token = Str::random(80);
        $this->storeToken($token, $email);
        return $token;
    }

    public function storeToken($token, $email){
        DB::table('password_resets')->insert([
            'email' => $email,
            'token' => $token,
            'created_at' => Carbon::now()
        ]);
    }

}
