<?php

namespace App\Http\Controllers;

use App\Models\Permission;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

class PermissionsController extends Controller
{
    public function index (Request $request): JsonResponse
    {
        try {
            $super_admin_permissions = [
                'all-tenants',
                'roles'
            ];
            $management = ['users', 'agent-profiles', 'groups', 'events-log'];
            $callCenter = [
                'call-center-features',
                'queue-selection',
            ];
            $misc = [
                'chat',
                'view-recordings',
                'update_social_networks',
            ];
            if ($request->user()->hasRole('super-admin')) {
                $permissions = Permission::all();
            } else {
                $permissions = Permission::where('slug', '!=', $super_admin_permissions)->get();
            }

            $groupedPermissions = [];
            foreach ($permissions as $key => $value) {
                if (in_array($value->slug, $management, true)) {
                    $groupedPermissions['management'][] = $value;
                } if (in_array($value->slug, $misc, true)) {
                    $groupedPermissions['misc'][] = $value;
                } if (in_array($value->slug, $callCenter, true)) {
                    $groupedPermissions['call-center'][] = $value;
                }
            }
            return response()->json(
                $groupedPermissions
            );
        } catch (Exception $exception) {
            return response()->json(
                compact('exception')
            );
        }
    }
}
