<?php

namespace App\Http\Controllers;

use App\Models\SocialNetwork;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;


//87343208
class SocialNetworks extends Controller
{
    public function index ($tenantId = null): JsonResponse
    {
        $userTenantId = auth()->user()->tenant_id;

        if (!auth()->user()->hasPermissionTo('all-tenants') && ($userTenantId != $tenantId) ) {
            return response()->json([
                'message' => __('not_allowed')
            ], 401);
        }
        if (!$tenantId) {
            $tenantId = $userTenantId;
        }
        $socials = SocialNetwork::where('tenant_id', $tenantId)->get();

        if (count($socials) < 1) {
            $socials = array(
                [
                    'tenant_id' => (int) $tenantId,
                    'network' => 'facebook',
                    'slug' => ''
                ],
                [
                    'tenant_id' => (int) $tenantId,
                    'network' => 'twitter',
                    'slug' => ''
                ],
                [
                    'tenant_id' => (int) $tenantId,
                    'network' => 'youtube',
                    'slug' => ''
                ],
                [
                    'tenant_id' => (int) $tenantId,
                    'network' => 'instagram',
                    'slug' => ''
                ]

            );
        }

        return response()->json(
            [
                'socials' => $socials
            ]
        );
    }

    public function update (Request $request) {

        $rules = [
            'social_networks' => 'required'
        ];

        $validator = Validator::make($request->all(), $rules);

        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 402);
        }

        $success = false;
        $data = [];

        foreach ($request->input('social_networks') as $social_network) {

            if (!auth()->user()->hasPermissionTo('all-tenants') && (auth()->user()->tenant_id != $social_network['tenant_id']) ) {
                break;
            }
            $social = SocialNetwork::where('network', $social_network['network'])
                ->where('tenant_id', $social_network['tenant_id'])
                ->first();

            if ($social) {
                $success = $social->update(['slug' => $social_network['slug']]);
                $payload [] = [
                    'network' => $social->network,
                    'slug' => $social->slug,
                    'tenant_id' => $social->tenant_id
                ];
            } else {
                if (auth()->user()->tenant_id) {
                    $success = SocialNetwork::create([
                        'tenant_id' => $social_network['tenant_id'],
                        'network' => $social_network['network'],
                        'slug' => $social_network['slug']
                    ]);
                }
            }
        }
        if ($success) {
            try {
                $eventLog = new PbxApiController();
                $user_aut = auth()->user();
                $eventLog->saveEvents(
                    $user_aut->tenant_id,
                    'Social Networks',
                    'update',
                    $payload,
                    $user_aut->user_name,
                    $user_aut->id,
                    [
                    ],
                    $request->ip(),
                    $request->userAgent()
                );
            } catch (Exception $e){
                // nothing to do here
            }

            return response()->json(
                [
                    'message' => __('social_networks_has_been_updated')
                ]
            );
        }

        return response()->json([
            'message' => __('has_not_been_possible_to_update_verify_your_data_and_permissions')
        ], 422);
    }
}
