<?php

namespace App\Http\Controllers;

use App\Models\Status;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;

class StatusController extends Controller
{

    public function index (): JsonResponse
    {
    	$status = Status::all();
    	return response()->json(compact('status'));
    }

    public function show (Status $status): JsonResponse
    {
        return response()->json(compact('status'));
    }

	public function store(Request $request): JsonResponse
    {
		try {
		    $input = $request->all();
			$status = Status::create($input);
		} catch (Exception $e) {
			return response()->json([
				'message' => $e->getMessage()
			], $e->getCode());
		}

        try {
            $eventLog = new PbxApiController();
            $user_aut = auth()->user();
            $eventLog->saveEvents(
                $user_aut->tenant_id,
                'Presence Status',
                'add',
                $request->except(['created_at', 'updated_at']),
                $user_aut->user_name,
                $user_aut->id,
                [
                ],
                $request->ip(),
                $request->userAgent()
            );
        } catch (Exception $e) {
            // nothing to do here
        }
		return response()->json([
			'message' => __('status_was_created'),
            'status' => $status
		]);
	}

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param \App\Models\Status $status
     * @return JsonResponse
     */
	public function update(Request $request, Status $status): JsonResponse
    {
        try {
            $default_status = [
                'available',
                'away',
                'do_not_disturb',
                'lunch_break',
                'business_trip'
            ];

            if (in_array($status->slug, $default_status, true)) {
                return response()->json(['message' => __('editing_default_presence_states_is_not_allowed')], 422);
            }

            $input = $request->all();
            $status->name = $input['name'];
            $status->icon = $input['icon'];
            $status->color = $input['color'];
            if( $status->update() ) {

                $eventLog = new PbxApiController();
                $user_aut = auth()->user();
                $eventLog->saveEvents(
                    $user_aut->tenant_id,
                    'Presence Status',
                    'update',
                    $request->except(['id','created_at','updated_at']),
                    $user_aut->user_name,
                    $user_aut->id,
                    [
                    ],
                    $request->ip(),
                    $request->userAgent()
                );

                return response()->json([
                    'message' => __('status_was_updated'),
                    'status' => $status
                ]);
            }

        } catch (Exception $e) {
            return response()->json(['error'], $e->getMessage());
        }
	}

	/**
	 * Remove the specified resource from storage.
	 *
	 * @param  Status  $status
	 *
     * @return JsonResponse
	 */
	public function destroy(Status $status)
	{
        try {
            $default_status = ['available', 'away', 'do_not_disturb', 'lunch_break', 'business_trip'];
            if (in_array($status->slug, $default_status)) {
                return response()->json(['message' => __('it_is_not_allowed_to_delete_the_default_presence_states')], 403);
            }
            $status->delete();

            $eventLog = new PbxApiController();
            $user_aut = auth()->user();
            $request = request();
            $eventLog->saveEvents(
                $user_aut->tenant_id,
                'Presence Status',
                'delete',
                [
                    'name' => $status->name,
                    'slug' => $status->slug,
                    'icon' => $status->icon,
                    'status color' => $status->color
                ],
                $user_aut->user_name,
                $user_aut->id,
                [
                ],
                $request->ip(),
                $request->userAgent()
            );

            return response()->json(['message' => __('status_was_deleted')]);
        } catch (\Exception $e) {
            return response()->json(['message'], $e->getMessage());
        }
	}
}
