<?php

namespace App\Http\Controllers;

use App\Models\UserSetting;
use Illuminate\Database\QueryException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use League\Flysystem\Exception;

class UserSettingsController extends Controller
{
    /**
     * Create a new AuthController instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth:api', ['except' => []]);
    }


	/**
	 * Display a listing of the resource.
	 *
	 * @return Response
	 */
	public function index()
	{
		//
	}

	/**
	 * Show the form for creating a new resource.
	 *
	 * @return Response
	 */
	public function create()
	{
		//
	}

	/**
	 * Store a newly created resource in storage.
	 *
	 * @return Response
	 */
	public function store()
	{
		//
	}

	/**
	 * Display the specified resource.
	 *
	 * @param  int  $id
	 * @return \Illuminate\Http\JsonResponse
     */
	public function show()
	{
        try {
            $settings = DB::table('user_settings')
                        ->where('user_id', '=', auth()->id())
                        ->get();
            return response()->json($settings);

        } catch (QueryException | Exception $e) {
            return response()->json([
                'message' => $e->getMessage()
            ], 422);
        }
	}

	/**
	 * Show the form for editing the specified resource.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function edit($id)
	{
		//
	}

	/**
	 * Update the specified resource in storage.
	 *
	 * @param Request $request
	 *
	 * @return \Illuminate\Http\JsonResponse
	 */
	public function update(Request $request)
	{
		try {
			$user = auth()->user();

            $this->validate($request, [
                'settings' => 'required',
            ]);

            $settings = $request->input('settings');

            foreach ($settings as $setting) {
                // check if user already has setting
                $user_settings = $user->settings()->where('type', '=', $setting['type'])->where('key', '=', $setting['key'])->first();

                if ($user_settings) {
                    $user_settings->value = $setting['value'];
                    // max answer timer: 25
                    if ($user_settings->key === 'auto_answer_timer' && ($setting['value'] < 0 || (int)$setting['value'] > 25)){
                        return response()->json(['error' => __('wrong_time_value')], 422);
                    }
                    $user->settings()->save($user_settings);
                } else {
                    // if the user does not have the setting, check that the setting exists in the default settings
                    foreach ( $user->defaultSettings() as $defaultSetting ) {
                        if ( $defaultSetting['type'] === $setting['type'] && $defaultSetting['key'] === $setting['key'] ) {
                            $user_settings = $defaultSetting;
                            break;
                        }
                    }
                    if (!$user_settings) {
                        return response()->json(['error' => __('user_setting_not_exist')], 404);
                    }
                    // add setting
                    $user_setting = new UserSetting();
                    $user_setting->type = $setting['type'];
                    $user_setting->key = $setting['key'];
                    $user_setting->value = $setting['value'];
                    $user->settings()->save($user_setting);
                }
            }
            // $user->with('status', 'groups', 'settings', 'roles','permissions')->where('id', $user->id)->first()
            return response()->json([
                'message' => __('user_settings_was_updated'),
                'user' => $user->load('settings')
            ], 200);

		} catch (Exception $e) {
			return response()->json(['error'], $e->getMessage());
		}
	}

	/**
	 * Remove the specified resource from storage.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function destroy($id)
	{
		//
	}
}
