<?php

namespace App\Http\Controllers;

use App\Http\Requests\UserSetupRequest;
use App\Models\Group;
use App\Models\Permission;
use App\Models\Role;
use App\Models\User;
use App\Models\UserSetting;
use App\Utils\AppStatus;
use Exception;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\Hash;

class UserSetupController extends Controller
{

    public function __construct()
    {
        /*$wizardCompleted = AppSetting::where('key', 'wizard_completed')->first();
        $installed = $wizardCompleted ? $wizardCompleted->value === '1' : false;*/

        $installed = AppStatus::isInstalled();

        if (!$installed) {
            $this->middleware(['auth:api', 'role:super-admin'])->except('setup');
        } else {
            $this->middleware(['auth:api', 'role:super-admin']);
        }
    }
    // setup first user
    public function setup (UserSetupRequest $request) {
        $validated = $request->validated();

        if ($validated['password'] !== $validated['passwordConfirmation']) {
            return response()->json(['error' => __('password_not_match')], 402);
        }

        try {
            /* Find default role */
            $superAdmin_role = Role::where('slug','super-admin')->first();
            /* Find default permissions */
            $tenants_perm = Permission::where('slug','all-tenants')->first();


            /* Default user */
            /**
             * Check if the user does not exist, if it exists, the information must be updated.
             */
            $user = User::where('user_name', '=', $validated['username'])->first();

            if (is_null($user)) {
                $user = User::where('email', '=', $validated['email'])->first();
            }

            if (!$user) {
                $superAdmin = new User();
                $superAdmin->name = $validated['name'];
                $superAdmin->user_name = $validated['username'];
                $superAdmin->email = $validated['email'];
                $superAdmin->tenant_id = '1';
                $superAdmin->presence = 'offline';
                $superAdmin->password = Hash::make($validated['password']);
                $superAdmin->save();
                /* Roles */
                $superAdmin->roles()->attach($superAdmin_role);
                /* Permissions */
                $superAdmin->permissions()->attach($tenants_perm);

                /* User settings */
                foreach ($superAdmin->defaultSettings() as $setting) {
                    $user_setting = new UserSetting();
                    $user_setting->type = $setting['type'];
                    $user_setting->key = $setting['key'];
                    $user_setting->value = $setting['value'];

                    if ($setting['key'] === 'timezone') {
                        $user_setting->value = $validated['timezone'];
                    }
                    $superAdmin->settings()->save($user_setting);
                }

                /* User status */
                $superAdmin->userStatus()->create([
                    'message' => '',
                    'status_id' => 1
                ]);
                /* User default Group */
                $default_group = Group::where('id', 1)->first();
                if ($default_group) {
                    $default_group->users()->attach($superAdmin->id);
                }
            } else {
                $user->name = $validated['name'];
                $user->user_name = $validated['username'];
                $user->email = $validated['email'];
                $user->password = Hash::make($validated['password']);
                /* User settings */
                $user_settings = $user->settings()->where('type', '=', 'general')
                                                  ->where('key', '=', 'timezone')->first();

                if ($user_settings) {
                    $user_settings->value = $validated['timezone'];
                    $user->settings()->save($user_settings);
                }
                $user->update();
            }

            Artisan::call('cache:clear');
            Artisan::call('config:cache');

            if (AppStatus::updateStatus('yes')) {
                /* AppSetting::where('key', 'wizard_completed')->update([
                    'value' => '1',
                ]); */

                return response()->json([
                    'message' => __('user_was_created')
                ]);
            } else {
                throw new Exception(__('an_unexpected_error_occurred_at_the_end_of_the_installation'), 422);
            }

        } catch (Exception $exception) {
            return response()->json([
                'test'=> AppStatus::updateStatus('yes'),
                'error' => $exception->getMessage()
            ]);
        }
    }
}
